#!/bin/sh
#
# organiza-cfd.sh
#
# Procesa un directorio conteniendo CFD's en formato XML o .zip y ejecuta
# procesa-cfd.sh con cada archivo.
#

FECHA="21may2014"
VERSION="2.3.1, $FECHA"


###################### Variables ################################

# Las variables deberian estar definidas desde $CONFIGDIR/organizafcd.conf
# pero si no, aqui ponemos unos defaults que puedan funcionar.

# Ruta al archivo de configuracion
CONFIGFILE="/etc/organizacfd/organizacfd.conf"

# Administrador a quien se enviaran los correos
ADMINMAIL="monitor@asic-linux.com.mx"

# Directorio de entrada
ORIGDIR=/datos/cf/entrada

# Directorio de log
LOGDIR="/var/log/cf"

# Ruta a procesa-cfd.sh
PROCESA_CFD="/usr/local/sbin/procesa-cfd.sh"

# Ruta a convmv
CONVMV=/usr/bin/convmv



#################################################################



###### Funciones utiles ######


error() {
   echo "*** $1"
   echo "$1" | mail -s "*** organiza-cfd error" $ADMINMAIL
   log_archivo "*** Error: $1"
   exit 1
}

log_archivo() {
   local logfile

   logfile="$LOGDIR/$(date +%Y%m%d)".txt
   echo "[ $(date +"%b %d %T") ] $1" | $UNIX2DOS >> "$logfile" ||  echo "Error escribiendo archivo de log" | mail -s "*** organiza-cfd error" $ADMINMAIL
}

###### Veamos si nos dieron todos los datos necesarios ######

if [ -z "$1" ]; then
   echo "Uso: `basename $0` [ -V | <archivo de configuracion> ]"
   exit 1
fi

if [ "$1" = "-V" ]; then
      echo "organiza-cfd.sh, version $VERSION"
      echo "Copyright 2011 asic-linux, S.A. de C.V."
      echo "Todos los derechos reservados."
      exit 0
   else
      CONFIGFILE="$1"
fi

# Leemos la configuracion
if [ -r "$CONFIGFILE" ]; then
   source "$CONFIGFILE"
else
   error "No puedo leer el archivo de configuracion $CONFIGFILE"
fi


###### Verificaciones de sanidad ######

[ -d "$ORIGDIR" ]  || error "El directorio de origen ($ORIGDIR) no existe o no es un directorio"
[ -x "$CONVMV" ]   || error "convmv ($CONVMV) no es ejecutable"

###### Bucle principal ######

# Dormimos unos segundos para dar tiempo a que aparezcan mas
# archivos en el directorio de entrada

sleep 5

# El RFC de mis CFD emitidos (Mayusculas)
MIRFC=$(echo "$MIRFC" | tr 'a-z' 'A-Z')

echo "[ $(date +"%b %d %T") ] Procesando directorio $ORIGDIR para RFC $MIRFC ($CONFIGFILE)"

# Convertimos los nombres de archivo del directorio de entrada a UTF-8
# La conversion es desde CP1252 (Windows LATIN1) porque es lo mas comun
# que vamos a encontrar.

$CONVMV -f cp1252 -t utf8 -r --notest "$ORIGDIR"


# Finalmente procesamos cada archivo
find $ORIGDIR -type f -print0 | xargs -0 -n1 -i $PROCESA_CFD "{}" "$CONFIGFILE" 


