#!/bin/sh
#
# procesa-buzon-cf.sh
#
# Uso:
# procesa-buzon-cf.sh <ruta al Maildir> <directorio de salida> [archivo de configuracion]
#
# Procesa los mensajes de correo almacenados en un buzon Maildir. Invoca
# a procesa-mensaje-cf.sh una vez por cada mensaje encontrado en el Maildir.
#

FECHA="21may2014"
VERSION="2.3.1"

###### Variables ######

# Ruta al archivo de configuracion
CONFIGFILE="/etc/organizacfd/procesabuzon.conf"


# Administrador a quien se enviaran los correos
ADMINMAIL=monitor@asic-linux.com.mx

# Ruta a procesa-menssaje-cf.sh
PROCESA_MENSAJE=/usr/local/sbin/procesa-mensaje-cf.sh

##### Funciones utiles #####

error() {
   echo "*** $1"
   echo "$1" | mail -s "*** procesa-buzon-cf error" $ADMINMAIL
   exit 1
}


# Verificamos que tengamos los argumentos necesarios


if [ "$1" = "-V" ]; then
      echo "procesa-buzon-cf.sh, version $VERSION"
      echo "Copyright 2011 asic-linux, S.A. de C.V."
      echo "Todos los derechos reservados."
      exit 0
fi

if [ -z "$2" ]; then
   echo "Uso: $(basename $0)  [ -V | <ruta al Maildir> <directorio de salida> [archivo de configuracion] ]"
   exit 1
fi

[ "$3" ] && CONFIGFILE="$3"

# Leemos la configuracion
if [ -r "$CONFIGFILE" ]; then
   source "$CONFIGFILE"
else
   error "No puedo leer el archivo de configuracion $CONFIGFILE"
fi

ORIGDIR="$1"
OUTDIR="$2"

# Verificaciones de sanidad
[ -d "$ORIGDIR" ]   || error "El Maildir ($ORIGDIR) no existe o no es un directorio"
[ -d "$OUTDIR" ]    || error "El directorio de salida ($OUTDIR) no existe o no es un directorio"
[ -d "$TMPBASE" ]   || error "El directorio temporal ($TMPBASE) no existe o no es un directorio"


# Exploremos el Maildir
find "$ORIGDIR" -type f | while read msg; do

   echo "Procesando $msg"
   $PROCESA_MENSAJE "$msg" "$OUTDIR" "$CONFIGFILE"
   echo

done

