#!/bin/sh
#
# procesa-cfd.sh
#
# Procesa archivos CFD en formato XML o .zip y los organiza:
#
# 1. Los separa en emitidos y recibidos
# 2. Los mueve a directorios representando el a~o y mes en que fueron emitidos
# 3. Cambia su nombre de acuerdo a este formato:
#
# xn_rs_fecha _cadena original.xml
#
# x: Serie
# n: Folio
# rs: Razon Social del cliente o proveedor
# (Para Emitidos o Recibidos, respectivamente)
# fecha: Fecha en formato AAAAMMDD
# orig: Nombre original del archivo
#
# Ej.
# Archivo original:
#    "CFD_WAAN56613.xml"
# Nuevo nombre:
#    "Nueva Wal Mart de Mexico, S de R.L. de C.V._2.1.1630_CFD_WAAN56613.xml"
#
#

FECHA="21may2014"
VERSION="2.3.1, $FECHA"

# Para ralentizar en caso de que haya muchos comprobantes
#sleep $(( $RANDOM % 180 ))


###################### Variables ################################

# Las variables deberian estar definidas desde $CONFIGDIR/organizafcd.conf
# pero si no, aqui ponemos unos defaults que puedan funcionar.

# Ruta al archivo de configuracion
CONFIGFILE="/etc/organizacfd/organizacfd.conf"

# Administrador a quien se enviaran los correos
ADMINMAIL="monitor@asic-linux.com.mx"

# Directorio de entrada
ORIGDIR=/datos/cf/entrada

# Directorio de salida
OUTDIR=/datos/cf/archivo

# Directorio para archivos que no pudimos clasificar
BADFILEDIR=/datos/cf/sin_clasificar

# Directorio para guardar los .zip que ya procesamos
ZIPDIR=/datos/cf/zip

# Directorio de log
LOGDIR="/var/log/cf"

# Ruta a xpath
XPATH="/usr/bin/xpath-1.13-7"

# Ruta a unix2dos o equivalente
UNIX2DOS=/usr/bin/todos

# Ruta a unzip
UNZIP=/usr/bin/unzip

# Ruta a procesa-cfd.sh
PROCESA_CFD="/usr/local/sbin/procesa-cfd.sh"

# El usuario al que pertenece el archivo
USUARIO_ARCHIVO="guest"

# Usamos una base de datos?
USABD=0

# Ruta a db-cfd.php
DBCFD=/usr/local/sbin/db-cfd.php



#################################################################


logfile="$LOGDIR/$(date +%Y%m%d)".txt

###### Funciones utiles ######

valorxml() {
      $XPATH -q -e "$1" "$2"  | cut -f2 -d\"
}

valorcfd() {
   local prefix
   local retval
   local ruta
   local archivo

   ruta="$1"
   archivo="$2"

   # Probamos con los prefijos de namespace en orden:
   # ninguno, cfd, cfdi

   for prefix in "" cfd: cfdi:; do
      rutatest=$(echo "$ruta" | sed "s/^/$prefix/g;s/\/\@/\@\@/g;s/\//\/$prefix/g;s/\@\@/\/\@/g")
      retval=$(valorxml "$rutatest" "$archivo")
      [ "$retval" ] && break # Salimos en cuanto obtengamos un valor
   done

   echo "$retval"
}

salida_novalor() {
   local msgnovalor

   msgnovalor="*** No pude determinar el valor \"$1\" ($cfd)"
   echo "$msgnovalor"
   # Si hubo errores al procesar este CFD lo almacenamos en
   # otro directorio para que sea revisado manualmente
   log_archivo "$msgnovalor"
   log_archivo "Moviendo $(basename "$cfd") a $BADFILEDIR"
   echo "Moviendo $(basename "$cfd") a $BADFILEDIR"
   mv -- "$cfd" "$BADFILEDIR" || error "Error moviendo el archivo $cfd a $BADFILEDIR"
   exit 2
}

error() {
   echo "*** $1"
   echo "$1" | mail -s "*** organiza-cfd error" $ADMINMAIL
   log_archivo "*** Error: $1"
   exit 1
}

log_archivo() {
   local logfile

   logfile="$LOGDIR/$(date +%Y%m%d)".txt
   echo "$(date +"%b %d %T") [$$] $1" | $UNIX2DOS >> "$logfile" ||  echo "Error escribiendo archivo de log" | mail -s "*** organiza-cfd error" $ADMINMAIL
}

###### Veamos si nos dieron todos los datos necesarios ######

if [ "$1" = "-V" ]; then
      echo "organiza-cfd.sh, version $VERSION"
      echo "Copyright 2011 asic-linux, S.A. de C.V."
      echo "Todos los derechos reservados."
      exit 0
   else
      CFDFILE="$1"
      CONFIGFILE="$2"
fi

if [ -z "$2" ]; then
   echo "Uso: `basename $0`  -V | <archivo> <archivo de configuracion>"
   exit 1
fi

# Leemos la configuracion
if [ -r "$CONFIGFILE" ]; then
   source "$CONFIGFILE"
else
   error "No puedo leer el archivo de configuracion $CONFIGFILE"
fi


###### Verificaciones de sanidad ######

[ "$MIRFC" ]       || error "El RFC de nuestros comprobantes no esta definido (Variable MIRFC en $CONFIGFILE)"
[ -x "$XPATH" ]    || error "xpath ($XPATH) no es ejecutable"
[ -x "$UNIX2DOS" ] || error "unix2dos ($UNIX2DOS) no es ejecutable"
[ -x "$UNZIP" ]    || error "unzip ($UNZIP) no es ejecutable"
[ -x "$DBCFD" ]    || error "db-cfd ($DBCFD) no es ejecutable"
[ -d "$OUTDIR" ]   || error "El directorio de salida ($OUTDIR) no existe o no es un directorio"
[ -d "$LOGDIR" ]   || error "El directorio de log ($LOGDIR) no existe o no es un directorio"
[ -d "$BADFILEDIR" ]  || error "El directorio de errores de procesamiento  ($BADFILEDIR) no existe o no es un directorio"
[ -d "$ZIPDIR" ]  || error "El directorio de archivos .zip procesados ($ZIPDIR) no existe o no es un directorio"
[ -f "$CFDFILE" ]  || error "El archivo a procesar ($CFDFILE) no existe o no es un archivo regular"

###### Bucle principal ######
unset LANG

# El RFC de mis CFD emitidos (Mayusculas)
MIRFC=$(echo "$MIRFC" | tr 'a-z' 'A-Z')

cfd="$CFDFILE"

   rutaemre="."
   prefijo="_"

   ZIPFILE=0
   MAILFILE=0
   # Verificamos que sea un archivo XML o ZIP o de correo
   if ! $( file "$cfd" | egrep -q "XML.*text"); then
      if $( file "$cfd" | egrep -q "Zip"); then
         ZIPFILE=1
      elif $( file "$cfd" | egrep -q "smtp mail" ); then
         MAILFILE=1
      else
         log_archivo "+++ Advertencia: \"$cfd\" no parece ser un documento XML ni un archivo .ZIP."
         log_archivo "+++ Advertencia: lo tratare como XML pero podria haber errores mas adelante."
      fi
   fi



# Si es un zip lo procesamos como tal
if [ $ZIPFILE -eq 1 ]; then

   if [ "$INSIDEZIP" ]; then
         log_archivo "Encontre .zip ($cfd) dentro de un archivo .zip. La recursion de archivos .zip esta prohibida."
   else

      # Procesamos cada .zip pero solo admitimos 1 nivel de recursion.
      # .zip dentro de .zip  esta prohibido
      export INSIDEZIP="1"

      # Generamos un directorio temporal
      TMPDIR=$( mktemp -d -p $TMPBASE procesacf.XXXXXXXXXX ) || error "Error al crear el directorio temporal bajo $TMPBASE"

      # Extraemos y movemos los xml que encontremos al directorio de entrada
      log_archivo "Extrayendo contenido de archivo ZIP ($cfd):"
      $UNZIP "$cfd" -d "$TMPDIR" 2>&1 > $TMPDIR/unziplog
      zipret=$?
      cat $TMPDIR/unziplog | while read ziplogline; do
         log_archivo "unzip: $( basename "$ziplogline")"
      done
      [ $zipret -gt 0 ] && log_archivo "*** Hubo un error al descomprimir el archivo $cfd"
      rm -f -- "$TMPDIR/unziplog"
#      find "$TMPDIR" -type f -name "*.xml" -o -name "*.XML" | xargs -n1 -i $PROCESA_CFD "{}" "$CONFIGFILE" 
      find "$TMPDIR" -type f -print0 | xargs -0 -n1 -i $PROCESA_CFD "{}" "$CONFIGFILE" 

      find "$TMPDIR" -type f -print0 | xargs -0 -i mv -v -- "{}" "$BADFILEDIR"

      [ -d "TMPDIR" ] && rm -r -- "$TMPDIR"

      mv -- "$cfd" "$ZIPDIR"
   fi

elif [ $MAILFILE -eq 1 ]; then

   # Los archivos .EML son movidos inmediatamente a sin_clasificar
   log_archivo "$(basename "$cfd") es un mensaje de correo. Moviendo a directorio de archivos sin clasificar."
   mv -- "$cfd" "$BADFILEDIR" || error "Error moviendo el archivo $cfd a $BADFILEDIR"

else

   # Si no, lo procesamos como CFD

   # Obtenemos los datos necesarios
   nombreorig=$(basename "$cfd")
   sufijomd5=$( md5sum < "$cfd" | awk '{print $1}' )
   serie=$(valorcfd "Comprobante/@serie" "$cfd")

   folio=$(valorcfd "Comprobante/@folio" "$cfd")
   if [ ! "$folio" ]; then
      log_archivo "+++ Advertencia: no pude determinar el folio de $(basename "$cfd")"
      folio="NNN"
   fi

   uuid=$(valorxml "/cfdi:Comprobante/cfdi:Complemento/tfd:TimbreFiscalDigital/@UUID" "$cfd")

   [ "$uuid" ] || uuid="SIN_UUID"

   # El Anexo 20 publicado por el SAT dice:
   # fecha
   # Descripción Atributo requerido para la expresión de la fecha y hora de
   # expedición del comprobante fiscal. Se expresa en la forma
   # aaaa-mm-ddThh:mm:ss, de acuerdo con la especificación ISO
   # 8601.
   #
   # Asi que vamos a escoger solo los primeros 19 caracteres de la fecha
   # ignorando la indicacion de desplazamiento de zona horaria, ejemplo:
   # Si Comprobante/fecha dice "2010-11-25T12:02:01-06:00", nosotros quitamos
   # el "-6:00" y nos quedamos con "2010-11-25T12:02:01"

   fechacfd=$(valorcfd "Comprobante/@fecha" "$cfd" | cut -c 1-19 )
   [ "$fechacfd" ] || salida_novalor "fecha"

   rfcemisor=$(valorcfd "Comprobante/Emisor/@rfc" "$cfd" | tr 'a-z' 'A-Z')
   [ "$rfcemisor" ] || salida_novalor "RFC del emisor"

   rfcreceptor=$(valorcfd "Comprobante/Receptor/@rfc" "$cfd" | tr 'a-z' 'A-Z')
   [ "$rfcreceptor" ] || salida_novalor "RFC del receptor"

   rs_emisor=$(valorcfd "Comprobante/Emisor/@nombre" "$cfd" | tr 'a-z/' 'A-Z_')
   [ "$rs_emisor" ] || salida_novalor "Razon Social del emisor"

   rs_receptor=$(valorcfd "Comprobante/Receptor/@nombre" "$cfd" | tr 'a-z/' 'A-Z_')
   [ "$rs_receptor" ] || rs_receptor="$rfcreceptor"


   anio=$(date -u -d "$fechacfd" +%Y)
   mes=$(date -u -d "$fechacfd" +%b)
   fecha=$(date -u -d "$fechacfd" +%Y%m%d)


   # Es emitido o recibido?
   if [ "$rfcemisor" = "$MIRFC" ]; then
      # Emitido
      rutaemre="emitidos"
      prefijo="${serie}${folio}_${rs_receptor}_${fecha}"

   elif [ "$rfcreceptor" = "$MIRFC" ]; then
         # Recibido
         rutaemre="recibidos"
         prefijo="${rs_emisor}_${serie}${folio}_${fecha}"
   else
         # Otros (Nuestro RFC no esta presente)
         rutaemre="otros"
         prefijo="${rs_emisor}_${serie}${folio}_${fecha}"
   fi

   # Creamos los directorios necesarios si no existen
   mdir="$OUTDIR/${rutaemre}/${anio}/${mes}"
   for creadir in "$OUTDIR/${rutaemre}" "$OUTDIR/${rutaemre}/${anio}"  "$mdir"; do
      if ! [ -d "$creadir" ]; then
         mkdir -p "$creadir" || error "Error creando el directorio $creadir"
         chown -- $USUARIO_ARCHIVO "$creadir";
      fi
   done

   # Registramos el CFD en la BD
   # $OUTDIR esta definido en $CONFIGFILE
   # $OUTDIR tipico: /datos/cf/usuario@comprobantefiscal.mx/archivo
   dbret=0
   dir_nombre="$(dirname "$cfd")"
   viejo_nombre="$(basename "$cfd")"

   if [ "$uuid" = "SIN_UUID" ]; then
      nuevo_nombre="${prefijo}_${sufijomd5}.xml"
   else
      nuevo_nombre="${prefijo}_${uuid}.xml"
   fi

  # Corregimos algunos caracteres que nos pueden causar problemas mas adelante
   nuevo_nombre=$(echo "$nuevo_nombre" | tr '/' '_') 


  # Lo movemos al nuevo nombre.
  # Nota que lo estamos haciendo en el mismo directorio entrada/ que esta
  # siendo monitoreado por incrond. Aqui estamos asumiendo que la entrada en
  # incrontab para el directorio incluye la opcion IN_NO_LOOP de forma que
  # no importa que hacemos el cambio de nombre ahi mismo porque IN_NO_LOOP
  # evitara que se vuelva a invocar a organiza-cfd.sh.

  mv -- "$cfd" "$dir_nombre/$nuevo_nombre"
  cfd="${dir_nombre}/$nuevo_nombre"

   usuariocf="$(basename $(dirname $OUTDIR))"
   if [ $USABD -eq 1 ]; then
      $DBCFD "$cfd" "$MIRFC" "$usuariocf" >>/tmp/cfdlog 2>&1
#      $DBCFD "$cfd" "$MIRFC" "$usuariocf" >>"$logfile" 2>&1
      dbret=$?
   fi

   if [ $dbret -gt 0 ]; then
      case $dbret in

         2) # RET_DUPLICADO
            log_archivo "Ya tenemos un comprobante con esta serie, folio, emisor y UUID. Ignorando."
            log_archivo "Serie: $serie, Folio: $folio, Emisor: $rfcemisor, Fecha: $fechacfd, UUID: $uuid"
            [ -d  "$ORIGDIR/../duplicados" ] || mkdir -p "$ORIGDIR/../duplicados"

            # Si lo movemos a duplicados lo dejamos con el nombre que tenia originalmente
            mv -- "$cfd" "$ORIGDIR/../duplicados/$viejo_nombre"
            ;;
         
         3) # RET_V3_NO_UUID
            log_archivo "El archivo $viejo_nombre dice ser un CFDi version 3.x pero no tiene un UUID."
            log_archivo "$viejo_nombre sera movido a documentos sin clasificar."
            [ -d  "$ORIGDIR/../sin_clasificar" ] || mkdir -p "$ORIGDIR/../sin_clasificar"

            # Si lo movemos a sin_clasificar lo dejamos con el nombre que tenia originalmente
            mv -- "$cfd" "$ORIGDIR/../sin_clasificar/$viejo_nombre"
            ;;

         *)
            log_archivo "Hubo un error al registrar el CFD $(basename "$cfd") en la base de datos."
            mv -- "$cfd" "$ORIGDIR/$viejo_nombre"
	    exit 1

      esac

   else
      # Movemos el archivo a su directorio de destino
      mv -v -- "$cfd" "$mdir/$nuevo_nombre" || error "Error moviendo el archivo a su destino"

      log_archivo "$(basename "$viejo_nombre") guardado en ${rutaemre}/${anio}/${mes}/${nuevo_nombre}"
   fi

fi

exit 0
