#!/bin/sh
#
# procesa-mensaje-cf.sh
#
# Uso:
# procesa-mensaje-cf.sh <archivo de correo> <directorio de salida> [archivo de configuracion]
#
# Procesa un mensaje de correo buscando archivos .xml o .zip incluidos como
# attachments, extrae los archivos y los copia a un directorio de salida.
#
# Si encuentra un .zip lo extrae en un directorio temporal y llama 
# recursivamente a si mismo para procesar su contenido.
#
# Si no encuentra archivos XML mueve el mensaje al directorio de salida
# poniendole extension .eml para que organiza-cfd.sh los ponga en el
# directorio sin_clasificar y ahi ya el usuario vea que hacerles.
#
# FIXME: Validar que los .xml sean CFD's
#
#


FECHA="21may2014"
VERSION="2.3.1"

###### Variables ######

# Ruta al archivo de configuracion
CONFIGFILE="/etc/organizacfd/procesabuzon.conf"

# Ruta a reformime
REFORMIME=/usr/local/bin/reformime

# Directorio temporal
TMPBASE=/var/tmp

# Administrador a quien se enviaran los correos
ADMINMAIL=monitor@asic-linux.com.mx

# Ruta a procesa-menssaje-cf.sh
PROCESA_MENSAJE=/usr/local/sbin/procesa-mensaje-cf.sh

# Borramos los mensajes despues de procesarlos?
BORRAMSG=1

# Movemos a OUTIDR los mensajes que no contienen archivos de interes?
MUEVEMSG=1

##### Funciones utiles #####

error() {
   echo "*** $1"
   echo "$1" | mail -s "*** procesa-buzon-cf error" $ADMINMAIL
   exit 1
}

advertencia() {
   local msgbasename

   echo "*** $1"
   (echo "$1"
    echo
    echo "Informacion del mensaje que procesamos:"
    echo
    egrep -i "^From:|^Date:|^Subject:" < "$msg"
    echo
   ) | mail -s "+++ procesa-buzon-cf advertencia" $ADMINMAIL

   if [ $MUEVEMSG -gt 0 ]; then
      echo "El mensaje sera movido a la bandeja de entrada de CFD para su procesamiento manual."
      msgbasename=$(basename "$msg")
      mv -v -- $msg ${OUTDIR}/${msgbasename}.eml || "Error moviendo archivo de correo a $OUTDIR"
      chmod -v -- 664 ${OUTDIR}/${msgbasename}.eml
      continue
   fi
}

# Verificamos que tengamos los argumentos necesarios


if [ "$1" = "-V" ]; then
      echo "procesa-buzon-cf.sh, version $VERSION"
      echo "Copyright 2011 asic-linux, S.A. de C.V."
      echo "Todos los derechos reservados."
      exit 0
fi

if [ -z "$2" ]; then
   echo "Uso: $(basename $0)  < -V | <archivo de correo> <directorio de salida> [archivo de configuracion] >"
   exit 1
fi

[ "$3" ] && CONFIGFILE="$3"

# Leemos la configuracion
if [ -r "$CONFIGFILE" ]; then
   source "$CONFIGFILE"
else
   error "No puedo leer el archivo de configuracion $CONFIGFILE"
fi

MSGFILE="$1"
OUTDIR="$2"

# Verificaciones de sanidad
[ -f "$MSGFILE" ]   || error "El mensaje ($MSGFILE) no existe o no es un archivo regular"
[ -d "$OUTDIR" ]    || error "El directorio de salida ($OUTDIR) no existe o no es un directorio"
[ -d "$TMPBASE" ]   || error "El directorio temporal ($TMPBASE) no existe o no es un directorio"
[ -x "$REFORMIME" ] || error "reformime ($REFORMIME) no es ejecutable"

# Generamos un directorio temporal
TMPDIR=$( mktemp -d -p $TMPBASE cfmsg.XXXXXXXXXX ) || error "Error al crear el directorio temporal bajo $TMPBASE"

# Procesemos el mensaje
   msg="$MSGFILE"

   echo "Procesando $msg"

   # Extraemos todos los attachments de este mensaje
   $REFORMIME -x"${TMPDIR}/" < "$msg" || advertencia "Error al extraer los archivos del mensaje $msg" 

   # Verificamos que haya .xml o .zip disponibles en el mensaje
   if [ -z "$(find "$TMPDIR" -name "*.xml" -o -name "*.XML" -o -name "*.zip" -o -name "*.ZIP")" ]; then
      advertencia "$msg no parece contener archivos XML o ZIP"
   else
      # Copiamos los .xml y .zip que encontremos al directorio de salida
      find "$TMPDIR" \( -name "*.xml" -o -name "*.XML" -o -name "*.zip" -o -name "*.ZIP" \) -print0 | xargs -0 -i mv -v -- "{}" "$OUTDIR" || advertencia "Error al mover los archivos XML / ZIP a $OUTDIR"


   fi

   RETVAL=$?


   if [ $BORRAMSG -gt 0 ]; then
      if [ $RETVAL -eq 0 ]; then
         rm -vf -- "$msg"
      fi
   fi

   # Limpiamos para extraer el que sigue
   rm -f -- $TMPDIR/*

   echo


[ -d "TMPDIR" ] && rm -r -- "$TMPDIR"
